function [masks,RegionStats,allOutlines] = AutoSeg(size1,size2,ceBinarySignMap,CellSizeThresh,CellAreaThresh,ZthreshFDR,MaxCellWidth,SmoZMap,NumDirs)

% AutoSeg
% a function that segments smoothed suprathreshold regions into regions corresponding to individual somata
% inputs:
% size1 = number of image rows
% size2 = number of image columns
% ceBinarySignMap = binary map in image space with smoothed significant regions = 1, subthreshold regions = 0
% CellSizeThresh = number of pixels corresponding to the smallest allowable cell
% CellAreaThresh = region area in terms of pixels above which it will be segmented into different cells
% ZthreshFDR = Z score corresponding to the FDR corrected significance threshold
% MaxCellWidth = maximum width in terms of pixels of cells found during segmentation
% SmoZMap = smoothed map in image space of Z scores 
% NumDirs = number of distinct directions for boundary searching
%
% outputs:
% masks = array with a mask matrix for each identified cell in which pixel corresponding to that cell = 1, others  = 0
% RegionsStats = a data struct with a list of pixels, centroid, and area for each cell
% allOutlines = a matrix the same size as each mask in which outlines of all identified cells are set to 1, other pixels are set to 0

% create a matrix based on this map with pixels from each distinct region equal to a distinct number
LabelMat = bwlabel(ceBinarySignMap);
RegionS = regionprops(LabelMat, 'PixelList','Centroid','Area');

%Find the Regions corresponding to individual somata, larger than CellSizeThresh and smaller than CellAreaThresh
rs = [];
for x = 1:length(RegionS)
    if (CellSizeThresh < RegionS(x).Area) && (RegionS(x).Area < CellAreaThresh);
        rs = [rs; x];
    end
end

masksSingle = zeros(size1,size2,length(rs),'uint8');
edgesSingle = zeros(size1,size2,length(rs));
for i = 1:length(rs)
    BW = roicolor(LabelMat,rs(i));
    masksSingle(:,:,i) = BW;
    edgesSingle(:,:,i) = edge(double(BW),'log');
end
edgeSum = sum(edgesSingle,3);
allOutlinesSingle = edgeSum;

% Establish the generic search trajectory segments, 16 emanating radially from a center
maxCellRad = round(MaxCellWidth/2); %calculate the max length for a segment
RadianMeasures = (2*pi/16)*(1:16);
RMsines = sin(RadianMeasures);
RMcosines = cos(RadianMeasures);
PixelTrajs.x = zeros(maxCellRad,NumDirs); %create zero arrays
PixelTrajs.y = zeros(maxCellRad,NumDirs);
for j = 1:maxCellRad %populate the arrays
    for k = 1:NumDirs
        PixelTrajs.x(j,k) = round(RMcosines(k)*j);
        if PixelTrajs.x(j,k) == 7
           PixelTrajs.x(j,k) = 6;
        end
        PixelTrajs.y(j,k) = round(RMsines(k)*j);
        if PixelTrajs.y(j,k) == 7
           PixelTrajs.y(j,k) = 6;
        end
        if PixelTrajs.x(j,k) == -5 && PixelTrajs.y(j,k) == -5
            PixelTrajs.x(j,k) = -6;
            PixelTrajs.y(j,k) = -6;
        end
    end
end

%Identify the regions larger than CellAreaThresh that need to be segmented
SupThreshRegs = find([RegionS(:).Area]>CellAreaThresh); 

%Create an array with masks for each large region
masksBig = zeros(size1,size2,length(SupThreshRegs));
for i = 1:length(SupThreshRegs)
    BW = roicolor(LabelMat,SupThreshRegs(i));
    masksBig(:,:,i) = BW;
end

%Now segment each large region and put the corresponding masks and mask edges into arrays
NewMasks = [];
NewMaskEdges = [];
h=waitbar(0,'Segmenting Large Regions'); %use a nifty waitbar to track progress
for i = 1:length(SupThreshRegs)
    %Create a map with one regions pixels set equal to their corresponding Z scores in the smoothed map, other pixels = 0
    threshSmoZMap = masksBig(:,:,i).*abs(SmoZMap); %take absolute value here to treat negative Z scores the same as positive Z scores 
    
    %Now create a similar smaller map with the same max dimensions as the region itself, plus a border of zero pixels one pixel thick 
    RemArea = RegionS(SupThreshRegs(i)).Area;
    RegYmin = min(RegionS(SupThreshRegs(i)).PixelList(:,2));
    RegYmax = max(RegionS(SupThreshRegs(i)).PixelList(:,2));
    RegXmin = min(RegionS(SupThreshRegs(i)).PixelList(:,1));
    RegXmax = max(RegionS(SupThreshRegs(i)).PixelList(:,1));
    RegMap = threshSmoZMap(RegYmin-1:RegYmax+1,RegXmin-1:RegXmax+1); %defines this smaller region map 
    
    %Now iterate through, segmenting out a regions until the remaining region is smaller than CellSizeThresh
    while RemArea > CellSizeThresh
        %Find the indices of the pixel corresponding to the maximum Z score
        [Cy,Iy] = max(RegMap);
        [Peak,Mx] = max(Cy);
        My = Iy(Mx);
        
        %Create SearchTrajs, which are the segments of PixelTrajs shifted to emanate from the pixel location just identified
        SearchTrajs.x = zeros(maxCellRad,NumDirs);
        SearchTrajs.y = zeros(maxCellRad,NumDirs);
        for m = 1:maxCellRad
            for n = 1:NumDirs
                if (PixelTrajs.x(m,n)+ Mx) > 0 && (PixelTrajs.x(m,n)+ Mx) <= size(RegMap,2)
                    SearchTrajs.x(m,n) = PixelTrajs.x(m,n)+ Mx;
                end
                if (PixelTrajs.y(m,n)+ My) > 0 && (PixelTrajs.y(m,n)+ My)<= size(RegMap,1)
                    SearchTrajs.y(m,n) = PixelTrajs.y(m,n)+ My;
                end
            end
        end
       
        %Now establish the bounds of the segmented region
        %Search trajectories are followed continue until:
        % 1. The next pixel in trajectory has a higher magnitude Z score
        % 2. The edge of the larger region is reached
        % 3. The end of the trajectory segment is reached
        BoundXs  = zeros(1,NumDirs+1); %zero arrays that will become the bounds of each segmented region
        BoundYs = zeros(1,NumDirs+1);
        for p = 1:NumDirs %iterate over each segment
            PrevCorr = Peak; %intially set the previous pixel's Z score to that of the center pixel
            FoundEdge = 0; %Signifies if edge has been found
            Bound = [0 0]; %Initial dummy bound for segment
            for q = 1:maxCellRad %Iterate over the length of the trajectory segment
                if SearchTrajs.x(q,p) > 0 && SearchTrajs.y(q,p) > 0 %check to make sure you haven't reached the edge of the larger region yet
                    CurrCorr = RegMap(SearchTrajs.y(q,p),SearchTrajs.x(q,p)); %Z score of the current pixel
                    if CurrCorr < ZthreshFDR || CurrCorr-PrevCorr > 0 % check if next pixel in trajectory has a higher magnitude Z score
                        if FoundEdge == 0
                            FoundEdge = 1; %if so, say the edge is found
                            Bound = [SearchTrajs.x(q,p) SearchTrajs.y(q,p)]; %and set the bounds according to the current pixel location
                        end
                    end
                    PrevCorr = CurrCorr; %update the correlation of the previous pixel as that of the current pixel
                end
                if  q < maxCellRad   
                    if SearchTrajs.y(q+1,p)== 0 || SearchTrajs.x(q+1,p)== 0 %check to see if you reach the edge of the larger region
                        if FoundEdge == 0
                            FoundEdge = 1; %if so, say the edge is found
                            Bound = [SearchTrajs.x(q,p) SearchTrajs.y(q,p)]; %and set the bounds according to the current pixel location
                        end
                    end
                end
            end
            if FoundEdge == 0 %if the edge still isn't found when you reach the last pixel in the trajectory, set the bounds as it
                Bound = [SearchTrajs.x(maxCellRad,p) SearchTrajs.y(maxCellRad,p)];
            end           
            
            %add the bounds for each segment to the overall set of bounds
            BoundXs(p) = Bound(1); 
            BoundYs(p) = Bound(2);
        end;
        
        %set an extra set of bound pixels as the same as the first; necessary for poly2mask to work properly
        BoundXs(end) = BoundXs(1);
        BoundYs(end) = BoundYs(1);
        
        %create a mask for the segmented region from the smaller map defined in line 76
        RegMask = poly2mask(BoundXs, BoundYs, size(RegMap,1),size(RegMap,2));
        
        %Now erase the segmented region from the larger regions
        invRegMask = ~RegMask;
        RegMap = invRegMask.*RegMap;
        
        %create a mask for the segmented region from the full map
        BoundXsWhole = BoundXs + (min(RegionS(SupThreshRegs(i)).PixelList(:,1))-2);
        BoundYsWhole = BoundYs + (min(RegionS(SupThreshRegs(i)).PixelList(:,2))-2);
        Mask = poly2mask(BoundXsWhole, BoundYsWhole, size1,size2);
        MaskEdge = edge(double(Mask),'log');
        MaskArea = sum(Mask(:));
        RemArea = RemArea-MaskArea;
        
        %if the mask is bigger than CellSizeThresh, you keep it and add it to the array NewMasks 
        if MaskArea > CellSizeThresh
            NewMasks = ([NewMasks; Mask]);
            NewMaskEdges = ([NewMaskEdges; MaskEdge]);
        end
    end
    waitbar(i/length(SupThreshRegs),h,'Segmenting Large Regions'); %update the waitbar to reflect progress
end
close(h); %close the waitbar

%To align mask diemsions as defined by poly2mask with the orientation of the data array, the dimensions of the masks need to be permuted
reNewMasks = reshape(NewMasks', size2,size1,size(NewMasks,1)/size1);
reNewMasks = permute(reNewMasks,[2 1 3]);

%Do the same for the mask edges
reNewMaskEdges = reshape(NewMaskEdges', size2,size1,(size(NewMaskEdges,1)/size1));
reNewMaskEdges = permute(reNewMaskEdges,[2 1 3]);

%define a matrix with all the outlines of the segmented masks
NewOutlines = sum(reNewMaskEdges,3);

%Now assemble an array with all segmented masks plus masksSingle (lines 29-35) 
masks = zeros(size(reNewMasks,1),size(reNewMasks,2),size(reNewMasks,3)+size(masksSingle,3),'uint8');
masks(:,:,1:size(masksSingle,3)) = masksSingle;
masks(:,:,(size(masksSingle,3)+1):size(masks,3)) = reNewMasks;
NewLabelMat = zeros(size(reNewMasks,1),size(reNewMasks,2));
NewLabelMat = uint8(NewLabelMat);
for z = 1:size(masks,3)
    NewLabelMat = NewLabelMat-NewLabelMat.*masks(:,:,z)+(masks(:,:,z)*z);
end

%Create a data struct "RegionStats" with a list of pixels, centroid, and area for each cell
RegionStats = regionprops(NewLabelMat, 'PixelList','Centroid','Area');

%Keep only those regions larger than CellSizeThresh
keep = [];
for w = 1:length(RegionStats)
    if RegionStats(w).Area > CellSizeThresh
        keep = [keep; w];
    end
end
RegionStats = RegionStats(keep);
allOutlines = uint8(allOutlinesSingle+NewOutlines);


