function bestFDRpThresh = FDRcontP(pVals,alpha,BSsamps,BSiters)

% bestFDRpThresh
% function to calculate a p value threshold generating a desired false discovery rate (FDR)
%
% inputs:
% pVals are the distribution of p values stemming from the significance testing prior to correction
% alpha = the desired FDR
% BSsamps = bootstrap samples used in FDR threshold calculation
% BSiters = bootstrap iterations used in FDR threshold calculation
%
% outputs:
% bestFDRpThresh = a p value threshold generating a desired false discovery rate (FDR)

pRange = [alpha alpha/3 alpha/10 alpha/30 alpha/100 alpha/300 alpha/1000 alpha/3000 alpha/10000 alpha/30000]; %the set of possible p value thresholds
l = length(pRange);
FDRestimates = zeros(1,l);

% Estimate the FDR for each p value threshold in pRange and find the threshold that ensures FDR <= alpha
%use a nifty waitbar to track progress
h=waitbar(0,'Calculating FDR-corrected p value threshold');
for i = 1:l
    FDRestimates(i) = calcFDR(pVals,pRange(i),BSsamps,BSiters); 
    waitbar(i/l,h,['Testing threshold ' num2str(i) ' out of ' num2str(l)]);
end
diffs = (alpha - FDRestimates);
currentMin = 1;
currentMinInd = 1;
for j = 1:length(diffs) % from the set of FDR estimates, find the largest one which is <= alpha
    if diffs(j)>0
        if diffs(j)<currentMin
            currentMin = diffs(j);
            currentMinInd = j;
        end
    end
end
bestFDRpThresh = pRange(currentMinInd);
%close the waitbar
close(h);
end

%Subfunction to calculate the FDR according to equation (10) in Storey 2002
function FDR = calcFDR(pVals,p,BSsamps,BSiters)
lambda = calcOptimalLambda(pVals,BSsamps,BSiters);
m = length(pVals);
W = sum(pVals>lambda);
R = sum(pVals<=p);
pi0 = W/((1-lambda)*m);
Prpg = max([R 1])/m;
FDR = pi0*p/Prpg;
end

%Subfunction to calculate the optimal lambda for FDR estimation using section 6 in Storey 2004
function lambda = calcOptimalLambda(pVals,BSsamps,BSiters)
rangeR = 0:0.05:0.95;
pi0s = zeros(1,length(rangeR));
MSE = zeros(1,length(rangeR));
for k = 1:length(rangeR)
    m = length(pVals);
    W = sum(pVals>rangeR(k));
    pi0s(k) = W/((1-rangeR(k))*m);
end
minLambda = min(pi0s);
for s = 1:length(rangeR)
    pi0BS = zeros(1,BSiters);
    for q = 1:BSiters
        RandInds = ceil(rand(1,BSsamps)*m);
        pValsBS = pVals(RandInds);
        WBS = sum(pValsBS>rangeR(s));
        pi0BS(q) = WBS/((1-rangeR(s))*BSsamps);
    end
    MSE(s) = (1/BSiters)*sum((pi0BS-minLambda).^2);
end
[c,lambdaInd] = min(MSE);
lambda = rangeR(lambdaInd);
end
        
    
    